@@ -72,7 +72,8 @@ using namespace KCalendarCore;
 static void fillFlightReservation(const QVector<QVariant> &reservations, const KCalendarCore::Event::Ptr &event);
 static void fillTrainReservation(const TrainReservation &reservation, const KCalendarCore::Event::Ptr &event);
 static void fillBusReservation(const BusReservation &reservation, const KCalendarCore::Event::Ptr &event);
-static void fillLodgingReservation(const LodgingReservation &reservation, const KCalendarCore::Event::Ptr &event);
+static void fillLodgingReservation(const QVector<QVariant> &reservations, const KCalendarCore::Event::Ptr &event);
+static void fillEvent(const KItinerary::Event &ev, const KCalCore::Event::Ptr &event);
 static void fillEventReservation(const QVector<QVariant> &reservations, const KCalendarCore::Event::Ptr &event);
 static void fillGeoPosition(const QVariant &place, const KCalendarCore::Event::Ptr &event);
 static void fillFoodReservation(const FoodEstablishmentReservation &reservation, const KCalendarCore::Event::Ptr &event);
@@ -83,7 +84,7 @@ static void fillTaxiReservation(const TaxiReservation &reservation, const KCalen
 QSharedPointer<KCalendarCore::Event> CalendarHandler::findEvent(const QSharedPointer<KCalendarCore::Calendar> &calendar, const QVariant &reservation)
 {
 #ifdef HAVE_KCAL
-    if (!JsonLd::canConvert<Reservation>(reservation) || !calendar) {
+    if (!(JsonLd::canConvert<Reservation>(reservation) || JsonLd::canConvert<KItinerary::Event>(reservation)) || !calendar) {
         return {};
     }
 
@@ -120,6 +121,22 @@ QVector<QVariant> CalendarHandler::reservationsForEvent(const QSharedPointer<KCa
 #endif
 }
 
+bool CalendarHandler::canCreateEvent(const QVariant &reservation)
+{
+#ifdef HAVE_KCAL
+    if (JsonLd::isA<FlightReservation>(reservation)) {
+        const auto f = reservation.value<FlightReservation>().reservationFor().value<Flight>();
+        if (f.departureTime().isValid() && f.arrivalTime().isValid()) {
+            return true;
+        }
+    }
+    return SortUtil::startDateTime(reservation).isValid();
+#else
+    Q_UNUSED(reservation);
+    return false;
+#endif
+}
+
 void CalendarHandler::fillEvent(const QVector<QVariant> &reservations, const QSharedPointer<KCalendarCore::Event> &event)
 {
     if (reservations.isEmpty()) {
@@ -133,13 +150,15 @@ void CalendarHandler::fillEvent(const QVector<QVariant> &reservations, const QSh
     if (typeId == qMetaTypeId<FlightReservation>()) {
         fillFlightReservation(reservations, event);
     } else if (typeId == qMetaTypeId<LodgingReservation>()) {
-        fillLodgingReservation(reservation.value<LodgingReservation>(), event);
+        fillLodgingReservation(reservations, event);
     } else if (typeId == qMetaTypeId<TrainReservation>()) {
         fillTrainReservation(reservation.value<TrainReservation>(), event);
     } else if (typeId == qMetaTypeId<BusReservation>()) {
         fillBusReservation(reservation.value<BusReservation>(), event);
     } else if (JsonLd::isA<EventReservation>(reservation)) {
         fillEventReservation(reservations, event);
+    } else if  (JsonLd::isA<Event>(reservation)) {
+        fillEvent(reservation.value<KItinerary::Event>(), event);
     } else if (JsonLd::isA<FoodEstablishmentReservation>(reservation)) {
         fillFoodReservation(reservation.value<FoodEstablishmentReservation>(), event);
     } else if (JsonLd::isA<RentalCarReservation>(reservation)) {
@@ -290,8 +309,9 @@ static void fillBusReservation(const BusReservation &reservation, const KCalenda
     event->setDescription(desc.join(QLatin1Char('\n')));
 }
 
-static void fillLodgingReservation(const LodgingReservation &reservation, const KCalendarCore::Event::Ptr &event)
+static void fillLodgingReservation(const QVector<QVariant> &reservations, const KCalendarCore::Event::Ptr &event)
 {
+    const auto reservation = reservations.at(0).value<LodgingReservation>();
     const auto lodgingBusiness = reservation.reservationFor().value<LodgingBusiness>();
 
     event->setSummary(i18n("Hotel reservation: %1", lodgingBusiness.name()));
@@ -301,16 +321,40 @@ static void fillLodgingReservation(const LodgingReservation &reservation, const
     event->setDtStart(QDateTime(reservation.checkinTime().date(), QTime()));
     event->setDtEnd(QDateTime(reservation.checkoutTime().date(), QTime()));
     event->setAllDay(true);
-    event->setDescription(i18n("Check-in: %1\nCheck-out: %2\nBooking reference: %3",
-                               QLocale().toString(reservation.checkinTime().time(), QLocale::ShortFormat),
-                               QLocale().toString(reservation.checkoutTime().time(), QLocale::ShortFormat),
-                               reservation.reservationNumber()));
     event->setTransparency(KCalendarCore::Event::Transparent);
+
+    QStringList desc;
+    if (reservation.checkinTime().isValid()) {
+        desc.push_back(i18n("Check-in: %1", QLocale().toString(reservation.checkinTime().time(), QLocale::ShortFormat)));
+    }
+    if (reservation.checkoutTime().isValid()) {
+        desc.push_back(i18n("Check-out: %1", QLocale().toString(reservation.checkoutTime().time(), QLocale::ShortFormat)));
+    }
+    if (!lodgingBusiness.telephone().isEmpty()) {
+        desc.push_back(i18n("Phone: %1", lodgingBusiness.telephone()));
+    }
+    if (!lodgingBusiness.email().isEmpty()) {
+        desc.push_back(i18n("Email: %1", lodgingBusiness.email()));
+    }
+    if (!lodgingBusiness.url().isEmpty()) {
+        desc.push_back(i18n("Website: %1", lodgingBusiness.url().toString()));
+    }
+
+    for (const auto &r : reservations) {
+        const auto reservation = r.value<LodgingReservation>();
+        const auto person = reservation.underName().value<KItinerary::Person>();
+        if (!person.name().isEmpty()) {
+            desc.push_back(person.name());
+        }
+        if (!reservation.reservationNumber().isEmpty()) {
+            desc.push_back(i18n("Booking reference: %1", reservation.reservationNumber()));
+        }
+    }
+    event->setDescription(desc.join(QLatin1Char('\n')));
 }
 
-static void fillEventReservation(const QVector<QVariant> &reservations, const KCalendarCore::Event::Ptr &event)
+static void fillEvent(const KItinerary::Event &ev, const KCalCore::Event::Ptr &event)
 {
-    const auto ev = reservations.at(0).value<EventReservation>().reservationFor().value<KItinerary::Event>();
     Place location;
     if (JsonLd::canConvert<Place>(ev.location())) {
         location = JsonLd::convert<Place>(ev.location());
@@ -320,7 +364,11 @@ static void fillEventReservation(const QVector<QVariant> &reservations, const KC
     event->setLocation(location.name());
     fillGeoPosition(location, event);
     event->setDtStart(ev.startDate());
-    event->setDtEnd(ev.endDate());
+    if (ev.endDate().isValid()) {
+        event->setDtEnd(ev.endDate());
+    } else {
+        event->setDtEnd(ev.startDate().addSecs(3600));
+    }
     event->setAllDay(false);
 
     if (ev.doorTime().isValid()) {
@@ -337,6 +385,12 @@ static void fillEventReservation(const QVector<QVariant> &reservations, const KC
             event->addAlarm(alarm);
         }
     }
+}
+
+static void fillEventReservation(const QVector<QVariant> &reservations, const KCalendarCore::Event::Ptr &event)
+{
+    const auto ev = reservations.at(0).value<EventReservation>().reservationFor().value<KItinerary::Event>();
+    fillEvent(ev, event);
 
     QStringList desc;
     for (const auto &r : reservations) {
