@@ -25,6 +25,7 @@
 
 #ifdef HAVE_KCAL
 #include <KCalendarCore/Calendar>
+#include <KCalendarCore/Event>
 #endif
 
 #include <KMime/Content>
@@ -35,6 +36,7 @@
 #include <QJsonArray>
 #include <QJsonDocument>
 #include <QJsonObject>
+#include <QMetaProperty>
 #include <QStandardPaths>
 
 using namespace KItinerary;
@@ -42,6 +44,7 @@ using namespace KItinerary;
 static void initResources() // must be outside of a namespace
 {
     Q_INIT_RESOURCE(extractors);
+    Q_INIT_RESOURCE(vdv_certs);
 }
 
 namespace KItinerary {
@@ -207,9 +210,8 @@ void ExtractorRepository::extractorsForBarcode(const QString &code, std::vector<
 }
 
 #ifdef HAVE_KCAL
-void ExtractorRepository::extractorsForCalendar(const QSharedPointer<KCalendarCore::Calendar> &cal, std::vector<Extractor> &extractors) const
+void ExtractorRepository::extractorsForCalendar(const KCalendarCore::Calendar *cal, std::vector<Extractor> &extractors) const
 {
-    std::vector<Extractor> v;
     for (auto it = d->m_extractors.begin(), end = d->m_extractors.end(); it != end; ++it) {
         for (const auto &filter : (*it).filters()) {
             if (filter.type() != ExtractorInput::ICal) {
@@ -224,6 +226,28 @@ void ExtractorRepository::extractorsForCalendar(const QSharedPointer<KCalendarCo
         }
     }
 }
+
+void ExtractorRepository::extractorsForEvent(const KCalendarCore::Event *event, std::vector<Extractor> &extractors) const
+{
+    for (auto it = d->m_extractors.begin(), end = d->m_extractors.end(); it != end; ++it) {
+        for (const auto &filter : (*it).filters()) {
+            if (filter.type() != ExtractorInput::ICal) {
+                continue;
+            }
+
+            const auto propIdx = KCalendarCore::Event::staticMetaObject.indexOfProperty(filter.fieldName().toUtf8().constData());
+            if (propIdx < 0) {
+                continue;
+            }
+            const auto prop = KCalendarCore::Event::staticMetaObject.property(propIdx);
+            const auto value = prop.readOnGadget(event);
+            if (filter.matches(value.toString())) {
+                ExtractorRepositoryPrivate::insertExtractor(*it, extractors);
+                break;
+            }
+        }
+    }
+}
 #endif
 
 void ExtractorRepository::extractorsForContent(const QString &content, std::vector<Extractor> &extractors) const
@@ -317,3 +341,28 @@ void ExtractorRepository::setAdditionalSearchPaths(const QStringList& searchPath
 {
     d->m_extraSearchPaths = searchPaths;
 }
+
+QJsonValue ExtractorRepository::extractorToJson(const Extractor &extractor) const
+{
+    QJsonArray a;
+    bool added = false;
+    for (const auto &e : d->m_extractors) {
+        if (e.fileName() != extractor.fileName()) {
+            continue;
+        }
+        if (extractor.name() == e.name()) {
+            a.push_back(extractor.toJson());
+            added = true;
+        } else {
+            a.push_back(e.toJson());
+        }
+    }
+    if (!added) {
+        a.push_back(extractor.toJson());
+    }
+
+    if (a.size() == 1) {
+        return a.at(0);
+    }
+    return a;
+}
