@@ -70,6 +70,16 @@ static double doubleValue(const QJsonValue &v)
 
 static QVariant propertyValue(const QMetaProperty &prop, const QJsonValue &v)
 {
+    // enum handling must be done first, as prop.type() == Int
+    if (prop.isEnumType() && v.isString()) {
+        auto value = v.toString();
+        if (value.startsWith(QLatin1String("http://schema.org/"))) {
+            value = value.mid(18);
+        }
+        const auto key = prop.enumerator().keyToValue(value.toUtf8().constData());
+        return key;
+    }
+
     switch (prop.type()) {
     case QVariant::String:
         return v.toString();
@@ -108,6 +118,8 @@ static QVariant propertyValue(const QMetaProperty &prop, const QJsonValue &v)
             return v.toDouble();
         }
         return v.toString().toInt();
+    case QVariant::Time:
+        return QTime::fromString(v.toString(), Qt::ISODate);
     case QVariant::Url:
         return QUrl(v.toString());
     default:
@@ -190,6 +202,7 @@ static QVariant createInstance(const QJsonObject &obj)
     MAKE_FACTORY(FlightReservation);
     MAKE_FACTORY(FoodEstablishment);
     MAKE_FACTORY(FoodEstablishmentReservation);
+    MAKE_FACTORY(LocalBusiness);
     MAKE_FACTORY(RentalCarReservation);
     MAKE_FACTORY(RentalCar);
     MAKE_FACTORY(ReserveAction);
@@ -210,28 +223,54 @@ static QVariant createInstance(const QJsonObject &obj)
     MAKE_FACTORY(TrainStation);
     MAKE_FACTORY(TrainTrip);
     MAKE_FACTORY(UpdateAction);
-    MAKE_FACTORY(ViewAction);    
+    MAKE_FACTORY(ViewAction);
+
+    if (type == QLatin1String("QDateTime")) {
+        auto dt = QDateTime::fromString(obj.value(QLatin1String("@value")).toString(), Qt::ISODate);
+        dt.setTimeZone(QTimeZone(obj.value(QLatin1String("timezone")).toString().toUtf8()));
+        return dt;
+    }
+
     return {};
 }
-
 #undef MAKE_FACTORY
 
+static QVector<QVariant> fromJson(const QJsonObject &obj) // TODO this should replace JsonLdDocument::fromJson once we can change the API
+{
+    const auto normalized = JsonLdImportFilter::filterObject(obj);
+    QVector<QVariant> result;
+    result.reserve(normalized.size());
+    for (const auto &n : normalized) {
+        const auto v = createInstance(n.toObject());
+        if (!v.isNull()) {
+            result.push_back(v);
+        }
+    }
+    return result;
+}
+
 QVector<QVariant> JsonLdDocument::fromJson(const QJsonArray &array)
 {
     QVector<QVariant> l;
     l.reserve(array.size());
     for (const auto &obj : array) {
-        const auto v = fromJson(obj.toObject());
-        if (!v.isNull()) {
-            l.push_back(v);
-        }
+        l.append(::fromJson(obj.toObject()));
     }
     return l;
 }
 
 QVariant JsonLdDocument::fromJson(const QJsonObject& obj)
 {
-    return createInstance(JsonLdImportFilter::filterObject(obj));
+    return fromJsonSingular(obj); // ### temporary, see above
+}
+
+QVariant JsonLdDocument::fromJsonSingular(const QJsonObject &obj)
+{
+    const auto normalized = JsonLdImportFilter::filterObject(obj);
+    if (normalized.isEmpty()) {
+        return {};
+    }
+    return createInstance(normalized.at(0).toObject());
 }
 
 static bool valueIsNull(const QVariant &v)
@@ -284,6 +323,8 @@ static QJsonValue toJsonValue(const QVariant &v)
             }
             return v.toDateTime().toString(Qt::ISODate);
         }
+        case QVariant::Time:
+            return v.toTime().toString(Qt::ISODate);
         case QVariant::Url:
             return v.toUrl().toString();
         case QVariant::Bool:
@@ -322,8 +363,12 @@ static QJsonValue toJsonValue(const QVariant &v)
 
         if (prop.isEnumType()) { // enums defined in this QMO
             const auto key = prop.readOnGadget(v.constData()).toInt();
-            const auto value = prop.enumerator().valueToKey(key);
-            obj.insert(QString::fromUtf8(prop.name()), QString::fromUtf8(value));
+            auto value = QString::fromUtf8(prop.enumerator().valueToKey(key));
+            // this is (ab)used elsewhere, so let's not interfere with enum serialization there for now
+            if (strncmp(mo->className(), "KItinerary::", 12) == 0) {
+                value = QLatin1String("http://schema.org/") + value;
+            }
+            obj.insert(QString::fromUtf8(prop.name()), value);
             continue;
         } else if (QMetaType::typeFlags(prop.userType()) & QMetaType::IsEnumeration) { // external enums
             obj.insert(QString::fromUtf8(prop.name()), prop.readOnGadget(v.constData()).toString());
