@@ -12,7 +12,7 @@
     License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+    along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "airportdbgenerator.h"
@@ -61,17 +61,24 @@ static bool soundsMilitaryish(const QString &s)
 static void stripAirportAllLanguages(QStringList &s)
 {
     // only languages used in the English (sic!) wikidata labels and description matter here
-    s.removeAll(QLatin1String("aeroport"));
-    s.removeAll(QLatin1String("aeroporto"));
-    s.removeAll(QLatin1String("aeropuerto"));
-    s.removeAll(QLatin1String("air"));
-    s.removeAll(QLatin1String("airfield"));
-    s.removeAll(QLatin1String("airpark"));
-    s.removeAll(QLatin1String("airport"));
-    s.removeAll(QLatin1String("airstrip"));
-    s.removeAll(QLatin1String("flughafen"));
-    s.removeAll(QLatin1String("lufthavn"));
-    s.removeAll(QLatin1String("terminal"));
+    s.removeAll(QStringLiteral("aeroport"));
+    s.removeAll(QStringLiteral("aeroporto"));
+    s.removeAll(QStringLiteral("aeropuerto"));
+    s.removeAll(QStringLiteral("air"));
+    s.removeAll(QStringLiteral("airfield"));
+    s.removeAll(QStringLiteral("airpark"));
+    s.removeAll(QStringLiteral("airport"));
+    s.removeAll(QStringLiteral("airstrip"));
+    s.removeAll(QStringLiteral("flughafen"));
+    s.removeAll(QStringLiteral("lufthavn"));
+    s.removeAll(QStringLiteral("terminal"));
+}
+
+static void normalizeAbbreviations(QStringList &l)
+{
+    for (auto &s : l) {
+        if (s == QLatin1String("intl")) { s = QStringLiteral("international"); }
+    }
 }
 
 void AirportDbGenerator::merge(Airport &lhs, const Airport &rhs)
@@ -107,7 +114,7 @@ bool AirportDbGenerator::fetchAirports()
 {
     // sorted by URI to stabilize the result in case of conflicts
     const auto airportArray = WikiData::query(R"(
-        SELECT DISTINCT ?airport ?airportLabel ?airportAltLabel ?iataCode ?icaoCode ?coord ?endDate ?demolished ?openingDate ?iataEndDate WHERE {
+        SELECT DISTINCT ?airport ?airportLabel ?airportAltLabel ?iataCode ?icaoCode ?coord ?endDate ?demolished ?officialClosure ?openingDate ?iataEndDate ?iataRank WHERE {
             ?airport (wdt:P31/wdt:P279*) wd:Q1248784.
             ?airport p:P238 ?iataStmt.
             ?iataStmt ps:P238 ?iataCode.
@@ -115,8 +122,10 @@ bool AirportDbGenerator::fetchAirports()
             OPTIONAL { ?airport wdt:P625 ?coord. }
             OPTIONAL { ?airport wdt:P582 ?endDate. }
             OPTIONAL { ?airport wdt:P576 ?demolished. }
+            OPTIONAL { ?airport wdt:P3999 ?officialClosure. }
             OPTIONAL { ?airport wdt:P1619 ?openingDate. }
             OPTIONAL { ?iataStmt pq:P582 ?iataEndDate. }
+            OPTIONAL { ?iataStmt wikibase:rank ?iataRank. }
             SERVICE wikibase:label { bd:serviceParam wikibase:language "en". }
         } ORDER BY (?airport))", "wikidata_airports.json");
     if (airportArray.isEmpty()) {
@@ -130,7 +139,9 @@ bool AirportDbGenerator::fetchAirports()
             continue;
         }
 
-        if (obj.contains(QLatin1String("endDate")) || obj.contains(QLatin1String("demolished")) || obj.contains(QLatin1String("iataEndDate"))) {
+        if (obj.contains(QLatin1String("endDate")) || obj.contains(QLatin1String("demolished"))
+            || obj.contains(QLatin1String("officialClosure")) || obj.contains(QLatin1String("iataEndDate"))
+            || obj.value(QLatin1String("iataRank")).toObject().value(QLatin1String("value")).toString().endsWith(QLatin1String("DeprecatedRank"))) {
             // skip closed airports or those with expired IATA codes
             continue;
         }
@@ -231,19 +242,49 @@ void KItinerary::Generator::AirportDbGenerator::indexNames()
         std::for_each(l.begin(), l.end(), [](QString &s) {
             s = s.toCaseFolded();
         });
+        normalizeAbbreviations(l);
+        std::sort(l.begin(), l.end());
         l.removeAll(it.value().iataCode.toCaseFolded());
         l.removeAll(it.value().icaoCode.toCaseFolded());
         stripAirportAllLanguages(l);
-        l.removeDuplicates();
-        for (const auto &s : l) {
-            if (s.size() <= 2) {
-                continue;
-            }
+        l.erase(std::unique(l.begin(), l.end()), l.end());
+        l.erase(std::remove_if(l.begin(), l.end(), [](const auto &s) { return s.size() <= 2; }), l.end());
+        for (const auto &s : qAsConst(l)) {
+            (*it).fragments.push_back(s);
             m_labelMap[s].push_back(it.value().iataCode);
         }
     }
-    for (auto it = m_labelMap.begin(); it != m_labelMap.end(); ++it) {
+    for (auto it = m_labelMap.begin(); it != m_labelMap.end();) {
         std::sort(it.value().begin(), it.value().end());
+
+        // TODO revisit this when we have experience with the higher level IATA code disambiguation in FlightPostProcessor
+#if 0
+        if (it.value().size() == 2) { // TODO generalize this from 2 to N
+            const auto lhsAirport = m_airportMap.value(m_iataMap.value(it.value().at(0)));
+            const auto rhsAirport = m_airportMap.value(m_iataMap.value(it.value().at(1)));
+
+            // if the fragments are exactly identical, this will never be unique
+            // removing the current fragment from the index is fine, but we can't do that elsewhere if the collision set is > 2
+            if (lhsAirport.fragments == rhsAirport.fragments) {
+                qDebug() << "Unresolvable index collision for" << lhsAirport.iataCode << lhsAirport.label << lhsAirport.uri << rhsAirport.iataCode << rhsAirport.label << rhsAirport.uri;
+                it = m_labelMap.erase(it);
+                continue;
+            }
+
+            // TODO check if one fragment set is a subset of the other, in which case the set difference
+            // can be used for the exclusion index
+            std::vector<QString> leftDiff, rightDiff;
+            std::set_difference(lhsAirport.fragments.begin(), lhsAirport.fragments.end(), rhsAirport.fragments.begin(), rhsAirport.fragments.end(), std::back_inserter(leftDiff));
+            std::set_difference(rhsAirport.fragments.begin(), rhsAirport.fragments.end(), lhsAirport.fragments.begin(), lhsAirport.fragments.end(), std::back_inserter(rightDiff));
+            if (leftDiff.empty() || rightDiff.empty()) {
+                qDebug() << "Subset fragments:" << leftDiff << rightDiff << lhsAirport.fragments << rhsAirport.fragments;
+            } else {
+                //qDebug() << "Overlapping fragments:" << lhsAirport.fragments << rhsAirport.fragments;
+            }
+        }
+#endif
+
+        ++it;
     }
 }
 
@@ -320,7 +361,7 @@ static const char name1_string_table[] =
     std::vector<Name1Index> string_offsets;
     string_offsets.reserve(m_labelMap.size());
     uint32_t label_offset = 0;
-    for (auto it = m_labelMap.begin(); it != m_labelMap.end(); ++it) {
+    for (auto it = m_labelMap.constBegin(); it != m_labelMap.constEnd(); ++it) {
         if (it.value().size() > 1) {
             continue;
         }
@@ -362,7 +403,7 @@ static const char nameN_string_table[] =
     stringN_offsets.reserve(m_labelMap.size() - string_offsets.size());
     uint16_t string_offset = 0;
     uint16_t iata_map_offset = 0;
-    for (auto it = m_labelMap.begin(); it != m_labelMap.end(); ++it) {
+    for (auto it = m_labelMap.constBegin(); it != m_labelMap.constEnd(); ++it) {
         if (it.value().size() == 1) {
             continue;
         }
@@ -381,7 +422,7 @@ static const uint16_t nameN_iata_table[] = {
     for (const auto &offset : stringN_offsets) {
         out->write("    ");
         for (const auto &iataCode : offset.iataList) {
-            out->write(QByteArray::number(std::distance(m_iataMap.begin(), m_iataMap.find(iataCode))));
+            out->write(QByteArray::number(std::distance(m_iataMap.constBegin(), m_iataMap.constFind(iataCode))));
             out->write(", ");
         }
         out->write(" // ");
